﻿var path = require('path');
var util = require('util');
var async = require('async');
var error = require("../util/error");
var verify = require("../util/verify");
var log = require("../util/log")(module);

var AsyncModelLoader = {

    // Internal worker. Loads a modelObject from a modelInfo.
    _load_item: function (modelInfo, callback) {
        // console.log('AsyncModelLoader._load_item.modelInfo', util.inspect(modelInfo, { depth: 3 }));
        if (modelInfo.contexts && modelInfo.contexts.length > 0) {

            // load contexts first

            // set production state on each context info
            modelInfo.contexts.forEach(function (ctxInfo) {
                ctxInfo.___state = modelInfo.___state;
                //NO log.info('Context.model.toload', ctxmodelInfo.srcFullPath().substr(modelInfo.___state.pman.options.basedir.length + 1));
            })

            async.map(modelInfo.contexts, AsyncModelLoader._load_item, function (err, contextObjects) {
                if (err)
                {
                    log.error('Error.AsyncModelLoader._load_item', util.inspect(err, { depth: null }));
                    callback(err, null);
                    return;
                }

                // prepare context object for model loading
                // if one context is a collection context prepare collection context

                var collectionContextModel = null,
                    collectionContextIndex = -1,
                    context = {};

                for (var i = 0; i < contextObjects.length; i++) {
                    // console.log('*** Context.model.loaded as', util.inspect(contextObjects[i], { depth: 1 }));
                    // log.info('Context.model.loaded as __' + contextObjects[i].WmtTag, modelInfo.contexts[i].srcFullPath().substr(modelInfo.___state.pman.options.basedir.length + 1));
                    var contextObject = contextObjects[i];
                    // console.log('contextObject.exportName', contextObject.___exportName);
                    context[contextObject.___exportName] = contextObject;
                    if (contextObject.___coll) {
                        // console.log('*+*+*+*+*+ AsyncModelLoader._load_item.contextObject.___modelRef', util.inspect(contextObject.___modelRef, { depth: 3 }));
                        collectionContextModel = contextObject;
                        collectionContextIndex = i;
                    }
                }
                // log.info('Context.model.loaded', modelInfo.contexts[0].srcFullPath().substr(modelInfo.___state.pman.options.basedir.length + 1));

                if (collectionContextIndex > -1) {

                    // One of the contextObjects is a collection context, load one modelObject for each item of the collection context
                    var collectionModelInfos = AsyncModelLoader.prepareCollectionModelInfos(modelInfo, contextObjects, collectionContextIndex, collectionContextModel);
                    async.map(collectionModelInfos, AsyncCollectionModelLoader.load, function (err, collModelObjects) {
                        if (err) { callback(err, null); return; }
                        callback(null, collModelObjects);
                    });

                } else {

                    // no collection context, load a single modelObject
                    var loader = modelInfo.___state.pman.getSchemaLoader(modelInfo.schema);
                    if (loader == null) throw error.error('Schema ' + modelInfo.schema + ' not found', modelInfo);
                    loader(modelInfo.srcFullPath(), context, function (err, modelObject) {
                        if (err) { callback(err, null); return; }
                        modelObject.contextObjects = contextObjects; // ???
                        modelObject.___exportName = modelInfo.exportName || modelInfo.schema;
                        modelObject.___coll = modelInfo.coll;
                        callback(null, modelObject);
                    });
                }
            });

        } else {

            // modelInfo has no context, we can check for a cached loaded model
            var srcFullPath = modelInfo.srcFullPath();

            // TODO debug cache error
            //    , modelObject = modelInfo.___state.models[srcFullPath];
            //if (verify.isObject(modelObject)) {
            //    log.warn("Model loaded from cache: " + srcFullPath);
            //    callback(null, modelObject);
            //    return;
            //}
            // log.warn("Model not found in cache: " + srcFullPath);

            // cached model not found, load model using the schema loader
            var loader = modelInfo.___state.pman.getSchemaLoader(modelInfo.schema);
            if (loader == null) throw error.error('Schema ' + modelInfo.schema + ' not found', modelInfo);
            loader(srcFullPath, {}, function (err, modelObject) {
                if (err) { callback(err, null); return; }
                modelObject.___exportName = modelInfo.exportName || modelInfo.schema;
                modelObject.___coll = modelInfo.coll;
                modelInfo.___state.models[srcFullPath] = modelObject;
                callback(null, modelObject);
            });

        }
    },

    prepareCollectionModelInfos: function (modelInfo, contexts, index, collContextModel) {

        var collContext = contexts[index];
        var collName = collContextModel.___coll.name;
        var collItemName = collContextModel.___coll.itemName;
        // console.log('collName, collItemName', collName, collItemName);
        if (!verify.isArray(collContext[collName]))
            throw error.error('TODO AsyncModelLoader.prepareCollectionModelInfos');
        var result = [];
        var coll = collContext[collName];
        coll.forEach(function (collItem) {
            var context = {};
            for (var i = 0; i < contexts.length; i++) {
                if (i == index) context[collItemName] = collItem;
                else context[contexts[i].___exportName] = contexts[i];
            }
            result.push({
                item: modelInfo,
                collItem: collItem,
                context: context
            });
        });
        return result;

    },

    // load a single modelObject from a modelInfo
    load: function (modelInfo, callback) {

        log.setLevel(modelInfo.___state.pman.options.verbose);
        log.info('Started.async.load.model', modelInfo.srcFullPath().substr(modelInfo.___state.pman.options.basedir.length + 1));
        var models = [modelInfo];
        async.map(models, AsyncModelLoader._load_item, function (err, result) {
            // console.log('*** loaded models', util.inspect(result, { depth: 1 }));
            if (err) { callback(err, null); return; }
            log.info('Ended.async.load.model', modelInfo.srcFullPath().substr(modelInfo.___state.pman.options.basedir.length + 1));
            callback(null, result[0]);
        });

    },

    // load a collection of modelObjects from a collection of modelInfos
    loadMany: function (modelInfos, callback) {

        if (modelInfos.length === 0) {
            callback(null, []);
            return;
        }

        log.setLevel(modelInfos[0].___state.pman.options.verbose);

        async.map(modelInfos, AsyncModelLoader._load_item, function (err, modelObjects) {
            if (err) { callback(err, null); return; }
            callback(null, modelObjects);
        });

    },

    // load the context objects of a model info
    loadContexts: function (modelInfo, callback) {

        log.setLevel(modelInfo.___state.pman.options.verbose);
        modelInfo.contexts.forEach(function (ctxitem) {
            ctxitem.___state = modelInfo.___state;
            //NO log.info('Context.model.toload', ctxitem.srcFullPath().substr(modelInfo.___state.pman.options.basedir.length + 1));
        })
        async.map(modelInfo.contexts, AsyncModelLoader._load_item, function (err, contextObjects) {
            // console.log('*** loaded contexts', util.inspect(result, { depth: 1 }));
            if (err) { callback(err, null); return; }
            callback(null, contextObjects);
        });

    }

};

var AsyncCollectionModelLoader = {

    load: function (collInfo, callback) {

        var loader = collInfo.item.___state.pman.getSchemaLoader(collInfo.item.schema);
        if (loader == null) throw error.error('Schema ' + collInfo.item.schema + ' not found', collInfo.item);
        loader(collInfo.item.srcFullPath(), collInfo.context, function (err, result) {
            if (err) { callback(err, null); return; }
            // No export name. This can only be a final model to be passed to a generator, not a context model.
            // result.___exportName = ...;
            result.___collItem = collInfo.collItem;
            callback(null, result);
        });

    }

}

module.exports = AsyncModelLoader;
